using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization;

namespace CommonClass
{
    [Serializable]
    public partial class WorkingSession : Form, ISerializable
    {
        private Dictionary<string, object> _extendData = new Dictionary<string, object>();
        private bool _isSaveInputFile = false;
        private bool _isInputFileModify = false;
        public bool IsInputFileModify
        {
            get 
            {
                return _isInputFileModify; 
            }
            set 
            {
                _isSaveInputFile = !value;
                _isInputFileModify = value; 
            }
        }
        private string _inputFilePath;
        protected IInputParser _inputParserFormat = null;
        protected IOutputParser _outputParserFormat = null;
        protected AlgorithmManager algorithmManager = null;
        private DataTable inputFileData = null;
        private DataTable outputFileData = null;

        public WorkingSession()
        {
            InitializeComponent();
            InitializeComponentAndData();
        }

        public WorkingSession(Dictionary<string, object> extendData)
        {
            InitializeComponent();            
            _extendData = extendData;                        
            InitializeComponentAndData();
            SetInOutFileTypes();
        }

        #region Events

        private void tscbInputTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            _inputParserFormat = SetInputParser(string.Format(".{0}", (string)((sender as ToolStripComboBox).ComboBox.SelectedItem as DataRowView)[1]));
        }

        private void tscbOutputTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            _outputParserFormat = SetOutputParser((string)((sender as ToolStripComboBox).ComboBox.SelectedItem as DataRowView)[1]);
        }
                                
        private void tsmiExit_Click(object sender, EventArgs e)
        {
            Exit();
        }

        private void tsbCheckSyntax_Click(object sender, EventArgs e)
        {
            CheckSyntaks();
        }
        
        private void tsbStartAlgorithm_Click(object sender, EventArgs e)
        {
            if (this._inputParserFormat.GetResult == null)
            {
                CheckSyntaks();
            }

            algorithmManager.StartAlgorithm();
        }

        private void btnBrowseInputFile_Click(object sender, EventArgs e)
        {
            openFile.Filter = SetupInputFileExtension();

            if (openFile.ShowDialog() == DialogResult.OK)
            {
                _inputFilePath = openFile.FileName;
                StreamReader reader = new StreamReader(openFile.FileName);
                _inputParserFormat = ProblemDefinition(reader.ReadToEnd(), Path.GetExtension(openFile.FileName));
                reader.Close();
                tbInputFilePath.Text = openFile.FileName;
                btnSaveInputFile.Enabled = true;
            }
        }

        private void btnBrowseOutputFile_Click(object sender, EventArgs e)
        {
            saveFile.Filter = SetupOuputFileExtension();
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                tbOutputFilePath.Text = saveFile.FileName;
                btnSaveResult.Enabled = true;
                try
                {
                    File.Create(saveFile.FileName);
                }
                catch(Exception err)
                {
                    MessageBox.Show(string.Format("{0}{1}{2}", err.Message, Environment.NewLine, "Pokuajte ponovno!"), "Greka");
                    saveFile = null;
                    saveFile = new SaveFileDialog();
                }
            }
        }
        private void btnSaveResult_Click(object sender, EventArgs e)
        {
            saveFile.Filter = SetupOuputFileExtension();

            if (string.IsNullOrEmpty(tbOutputFilePath.Text))
            {
                MessageBox.Show("Odaberite izlaznu datoteku!");
                return;
            }

            if (_outputParserFormat != null)
            {
                if (algorithmManager.OutputData != null)
                {
                    _outputParserFormat.SetResults = algorithmManager.OutputData;
                    _outputParserFormat.WriteToFile(tbOutputFilePath.Text);
                }
            }
            else
                MessageBox.Show("Ne postoje podaci koji se mogu spremiti!");
        }

        private void WorkingSession_FormClosing(object sender, FormClosingEventArgs e)
        {
           DialogResult dr = MessageBox.Show("elite li spremiti projekt?", "Upozorenje", MessageBoxButtons.YesNoCancel);

           if (dr == DialogResult.Yes)
           {
               SaveProject();
           }
           else if (dr == DialogResult.Cancel)
           {
               e.Cancel = true;
           }            
        }

        private void New_Click(object sender, EventArgs e)
        {
            if (!_isInputFileModify)
            {
                MakeNewProblem();
                _isInputFileModify = true;
                tbInputFilePath.Text = "";
                _inputParserFormat = null;
                return;
            }

            if (_isSaveInputFile)
            {
                MakeNewProblem();
            }
            else
            {
                if(SaveInputFile())
                    MakeNewProblem();
            }
            tbInputFilePath.Text = "";
            _inputFilePath = "";
            _isInputFileModify = false;
            _inputParserFormat = null;
        }

        private void Save_Click(object sender, EventArgs e)
        {
            SaveInputFile();
        }

        private void tsmiSaveProject_Click(object sender, EventArgs e)
        {
            SaveProject();
        }

        private void tsmiLoadProject_Click(object sender, EventArgs e)
        {
            LoadProject();
        }

        #endregion

        #region Methods

        protected void ShowAlgorithmManager()
        {
            if (algorithmManager != null)
            {
                algorithmManager.Dock = DockStyle.Fill;
                this.pAlgorithmManager.Controls.Add(algorithmManager);
            }
        }

        private void SetInOutFileTypes()
        {
            inputFileData = new DataTable();
            inputFileData.Columns.Add("InputParserDescription", typeof(string));
            inputFileData.Columns.Add("InputParserName", typeof(string));

            outputFileData = new DataTable();
            outputFileData.Columns.Add("OutputParserDescription", typeof(string));
            outputFileData.Columns.Add("OutputParserName", typeof(string));

            tscbInputTypes.ComboBox.BindingContext = this.BindingContext;
            tscbInputTypes.ComboBox.DataSource = SetInputFileTypes(inputFileData, "InputParserDescription", "InputParserName");
            tscbInputTypes.ComboBox.DisplayMember = "InputParserDescription";
            tscbInputTypes.ComboBox.ValueMember = "InputParserName";

            tscbOutputTypes.ComboBox.BindingContext = this.BindingContext;
            tscbOutputTypes.ComboBox.DataSource = SetOutputFileTypes(outputFileData, "OutputParserDescription", "OutputParserName");
            tscbOutputTypes.ComboBox.DisplayMember = "OutputParserDescription";
            tscbOutputTypes.ComboBox.ValueMember = "OutputParserName";
        }

        private void SaveFile()
        {
            if (File.Exists(_inputFilePath))
                File.Delete(_inputFilePath);

            File.WriteAllText(_inputFilePath, ProblemToString());
        }

        private bool SaveInputFile()
        {
            if (MessageBox.Show("elite li spremiti ulaznu datoteku?", "Upozorenje", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                if (string.IsNullOrEmpty(_inputFilePath))
                {
                    saveFile.Filter = SetupInputFileExtension();
                    if (saveFile.ShowDialog() == DialogResult.OK)
                    {
                        _inputFilePath = saveFile.FileName;
                        SaveFile();
                        tbInputFilePath.Text = saveFile.FileName;
                        _isSaveInputFile = true;
                    }
                }
                else
                {
                    SaveFile();
                    _isSaveInputFile = true;
                }
                return true;
            }
            return false;
        }

        private void LoadProject()
        {
            try
            {
                openFile.Filter = string.Format("Optimization Framework (*{0}) | *{0}", Path.GetExtension((string)this._extendData["saveProject"]));
                if (openFile.ShowDialog() == DialogResult.OK)
                {
                    WorkingSession ws = ProjectSerializer.LoadProject(openFile.FileName);
                    ws.MdiParent = this.MdiParent;
                    this.Exit();
                    ws.Show();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(string.Format("Projekt se nemoe uitati!{0}Poruka:-->{1}{2}Satck-->{3}", Environment.NewLine, e.Message, Environment.NewLine, e.StackTrace), "Greka", MessageBoxButtons.OK);
            }
        }

        private void SaveProject()
        {
            try
            {
                ProjectSerializer.SaveProject((string)this._extendData["saveProject"], this);
            }
            catch (Exception e)
            {
                MessageBox.Show(string.Format("Projekt se nemoe spremiti!{0}Poruka:-->{1}{2}Satck-->{3}", Environment.NewLine, e.Message, Environment.NewLine, e.StackTrace), "Greka", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Funkciju poziva event za provjeru sintakse
        /// </summary>
        protected void CheckSyntaks()
        {
            try
            {
                if (_inputParserFormat != null)
                {
                    if (string.IsNullOrEmpty(SetInputText()))
                    {
                        System.Windows.Forms.MessageBox.Show("Upiite problem!", "Upozorenje", System.Windows.Forms.MessageBoxButtons.OK);
                        return;
                    }
                    BeforeCheckSyntaks();
                    _inputParserFormat.SetInputText = SetInputText();
                    _inputParserFormat.Start();
                    algorithmManager.InputData = _inputParserFormat.GetResult;
                    AfterSuccessCheckSyntaks();
                    rtbOutput.AppendText("Uspjeno parsirana ulazna datoteka!\n");
                }
                else
                    MessageBox.Show("Odaberite ulaznu datoteku ili odaberite tip ulazne datoteke!", "Upozorenje");
            }
            catch (ParserException err)
            {
                rtbOutput.AppendText(string.Format("{0}\n", err.Message));
            }
        }

        protected virtual void Exit()
        {
            this.Close();
        }


        protected string GetDataFromFile(string path)
        {
            System.IO.StreamReader sr = new System.IO.StreamReader(path);
            return sr.ReadToEnd();
        }

        #endregion

        #region ISerializable Members
        public WorkingSession(SerializationInfo serInfo, StreamingContext streamContext)
        {
            InitializeComponent();
            InitializeComponentAndData();
            this._inputParserFormat =
                serInfo.GetValue("inputParser", typeof(IInputParser)) != null ? (IInputParser)serInfo.GetValue("inputParser", typeof(IInputParser)) : null;
            this._outputParserFormat =
                serInfo.GetValue("outputParser", typeof(IOutputParser)) != null ? (IOutputParser)serInfo.GetValue("outputParser", typeof(IOutputParser)) : null;
            this._extendData =
                serInfo.GetValue("extendData", typeof(Dictionary<string, object>)) != null ? (Dictionary<string, object>)serInfo.GetValue("extendData", typeof(Dictionary<string, object>)) : null;
            this.rtbOutput.Text = (string)serInfo.GetValue("outputText", typeof(string));
            this.tbInputFilePath.Text =
                serInfo.GetString("inputParserPath");
            this.tbOutputFilePath.Text =
                serInfo.GetString("outputParserPath");

            this.outputFileData =
                serInfo.GetValue("outputFileData", typeof(DataTable)) != null ? (DataTable)serInfo.GetValue("outputFileData", typeof(DataTable)) : null;
            this.inputFileData =
                serInfo.GetValue("inputFileData", typeof(DataTable)) != null ? (DataTable)serInfo.GetValue("inputFileData", typeof(DataTable)) : null;
            if (outputFileData != null && inputFileData != null)
                SetInOutFileTypes();

            this.algorithmManager = serInfo.GetValue("algorithmManager", typeof(AlgorithmManager)) != null ? serInfo.GetValue("algorithmManager", typeof(AlgorithmManager)) as AlgorithmManager : null;

            ShowAlgorithmManager();
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("inputParser", _inputParserFormat);
            info.AddValue("outputParser", _outputParserFormat);
            info.AddValue("extendData", _extendData);
            info.AddValue("outputText", rtbOutput.Text);
            info.AddValue("inputParserPath", tbInputFilePath.Text);
            info.AddValue("outputParserPath", tbOutputFilePath.Text);

            info.AddValue("inputFileData", inputFileData);
            info.AddValue("outputFileData", outputFileData);

            info.AddValue("algorithmManager", algorithmManager);
        }
        #endregion

        #region Ovreride Methods

        protected virtual void InitializeComponentAndData() { }
        protected virtual void RegisterFinishedAlgorithmEvent() { }
        protected virtual DataTable SetInputFileTypes(DataTable data, string description, string name) { throw new Exception("Implementiraj metodu SetInputFileTypes"); }
        protected virtual DataTable SetOutputFileTypes(DataTable data, string description, string name) { throw new Exception("Implementiraj metodu SetOutputFileTypes"); }
        protected virtual string SetInputText() { return string.Empty; }
        protected virtual void AfterSuccessCheckSyntaks() { }
        protected virtual void BeforeCheckSyntaks() { }
        protected virtual string SetupInputFileExtension() { return string.Empty; }
        protected virtual IInputParser ProblemDefinition(string inputText, string selectedExtension) { return null; }
        protected virtual string SetupOuputFileExtension() { return string.Empty; }
        protected virtual IOutputParser SetOutputParser(string outputParserType) { return null; }
        protected virtual IInputParser SetInputParser(string inputParserType) { return null; }
        protected virtual void MakeNewProblem() { throw new Exception("Implementiraj metodu >>MakeNewProblem<<"); }
        protected virtual string ProblemToString() { return string.Empty; }

        #endregion
    }
}